
prism.run(["eucalyptus.services.metaService","$http", function (metaService,$http) {

	prism.custom = {
		metaService: metaService,
		http: $http
	};

}])

/*	
Bookmark Structure
bookmark = {
	type: "bookmark",
	dashboardId: "dash id",
	createdBy: "user email",
	isPublic: true,
	filters: {}
}

*/

prism.run([function(){

	//	Initialize the bookmarks object
	prism.bookmarks = {
		saved: {},
		popup: function() {
			
			//	Get the scope
			var myScope = prism.$ngscope;
			
			//	Get the dom injector
			var myDom = prism.$injector.get('ux-controls.services.$dom');

			//	Define the modal window's options
			var options = {
				scope: { 
					in: myScope 
				},
				templateUrl: '/plugins/bookmarks/bookmarkTemplate.html'				
			};

			//	Define the ui options			
			var ui = {};

			//	Refresh the list of saved bookmarks
			prism.bookmarks.initBookmarks();
			//prism.bookmarks.saved = prism.activeDashboard.settings ? prism.activeDashboard.settings.bookmarks : {};

			//	Add the modal window			
			myDom.modal(options,ui);
			
			//	Apply the scope, but make sure  
			
			if (!myScope.$$phase){
				myScope.$apply();
			}
		},
		loadBookmark: function(bookmark) {
			
			//	Get the new filter selections
			var filters = bookmark.filters;			

			//	Define options for all filters
			var options = {
				save:true,
				refresh:false,
				unionIfSameDimensionAndSameType:false
			};
			
			//	Make sure the dashboard is not auto-updating on each filter change
			var isAutoUpdating = false;
			if (prism.activeDashboard.autoUpdateOnFiltersChange) {
				prism.activeDashboard.autoUpdateOnFiltersChange = false;
				isAutoUpdating = true;
			}

			//	Clear all existing filters
			prism.activeDashboard.filters.clear();
			
			//	Loop through and add all filters
			$.each(filters,function(){
				//	Add this filter individually
				prism.activeDashboard.filters.update(this,options);
			})

			//	Reset the auto-updating setting
			if (isAutoUpdating) {
				prism.activeDashboard.autoUpdateOnFiltersChange = true;
			}

			//	Update the dashboard
			prism.activeDashboard.refresh();
		},
		saveBookmark: function(bookmark) {
			
			//	Create a bookmark key	
			var key = bookmark.createdBy + '-' + bookmark.name;

			//	Is this updating an existing bookmark?
			var bookmarkIsNew = (prism.bookmarks.saved) && (typeof prism.bookmarks.saved[key] == "undefined");

			//	API Endpoint
			var url = bookmarkIsNew ? '/api/v1/custom_data' : '/api/v1/custom_data/query';

			//	Define the method, depending on whether its a new bookmark or updating existing
			var method = bookmarkIsNew ? "POST" : "PATCH";

			//	Make API call
			$.ajax({
				url: url,
				method: method,
				contentType:'application/json',
				data:JSON.stringify(bookmark)
			}).done(function(data){
				//	write the saved bookmarks to the dashboard's settings property
				//prism.bookmarks.saved = data.bookmarks;

				//	Add to the existing list of bookmarks
				var key = data.createdBy + '-' + data.name;
				if (!prism.bookmarks.saved){
					prism.bookmarks.saved = {}
				}
				prism.bookmarks.saved[key] = data;
			});
		},
		deleteBookmark: function(bookmark,scope) {

			//	Create a bookmark key	
			var bookmarkId = bookmark._id;
			
			//Create the key to delete
			var query = {
				type:"bookmark",
				createdBy: bookmark.createdBy,
				isPublic: bookmark.isPublic,
				dashboardId: bookmark.dashboardId,
				name: bookmark.name
			};			

			//	Delete via API
			var url = '/api/v1/custom_data/query';

			//	Make API call
			$.ajax({
				url: url,
				method: "DELETE",
				contentType:'application/json',
				data:JSON.stringify(query)
			}).then(function(data){ 
				//	Delete from local list
				for (var i=prism.bookmarks.saved.length-1; i>=0; i--){
					var isMatch = prism.bookmarks.saved[i]._id == bookmarkId;
					if (isMatch){
						prism.bookmarks.saved.splice(i,1)		
					}
				}
				//	Make sure the modal window is updated
				scope.$apply()
			})
		},
		initBookmarks:function() {

			//	API Endpoint
			//var url = '/api/v1/dashboards/' + prism.activeDashboard.oid + '?fields=bookmarks';
			
			//	Define the query to use			
			var query = {
				type:"bookmark",
				dashboardId: prism.activeDashboard.oid,
				$or:[
					{
						createdBy:prism.user.email
					},{
						isPublic:true
					}
				]
			}
			//	Create the API endpoint
			var url = '/api/v1/custom_data/query?query=' + encodeURIComponent(JSON.stringify(query));
			
			//	Make API call
			$.ajax({
				url: url,
				method: 'GET',
				async: false,
				contentType:'application/json'
			}).done(function(data){
				//	Make sure the dashboard has a settings object
				if (typeof prism.bookmarks.saved === "undefined") {
					prism.bookmarks.saved = {}
				};
				//	write the saved bookmarks to the dashboard's settings property
				prism.bookmarks.saved = data;
			});
		}
	}

	//	Define the controller for the modal window
	mod.controller('bookmarksController', ['$scope',  function ($scope) {

	    	//	Grab the scope
	    	var myScope = $scope;	    	
	    	
	    	//	Load the current bookmarks
	    	myScope.bookmarks = prism.bookmarks.saved ? prism.bookmarks.saved : {};	

	    	//	Create variable for the bookmark name check
	    	myScope.newBookmarkNameIsInvalid = false;

	    	//	Define function to clean up properties at each level of a filter
	    	var cleaner = function(object) {

	    		//	Loop through each property at this level	    		
	    		for (key in object){

	    			//	Delete any properties that start with $
    				var startsWithDollar = (key.substring(0,1) === '$');

    				//	Delete functions
    				var isFunction = (typeof object[key] === 'function');

    				//	Should this property be deleted
    				if (startsWithDollar || isFunction) {
    					delete object[key];
    				} else {
    					//	Keeping the property, but does it have children?
		    			if (object.hasOwnProperty(key) && !$.isEmptyObject(object[key]) && (typeof object[key] === 'object')) {
		    				//	Run function recursively
		    				cleaner(object[key]);
		    			}
    				}
	    		}
	    	}

	    	//	Define a function to clean up the filter objects
	    	var getFilters = function() {

	    		//	Get the dashboard filters
	    		var dashboardFilters = prism.activeDashboard.filters.$$items;
	    		var savedFilters = [];

	    		//	Loop through all the filters
	    		$.each(dashboardFilters,function(){

	    			//	Create a clean filter object	    			
	    			var thisFilter = $.extend({},this);	    			
	    			cleaner(thisFilter);

	    			//	Save the filter
	    			savedFilters.push(thisFilter);
	    		})

	    		//	Return the clean list of filters
	    		return savedFilters;
	    	}

	    	//	Define new bookmark
	    	myScope.newBookmark = {
	    		type: "bookmark",
				dashboardId: prism.activeDashboard.oid,
				createdBy: prism.user.email,
				isPublic: false,
				filters: [],
	    		name: ''
	    	};

	    	//	Set the current filters
	    	myScope.newBookmark.filters = getFilters();

	    	//	Variable to watch, for determining if the bookmark's name is valid
	    	myScope.isValidName = false;

	    	//	Function to validate the bookmark name
	    	myScope.validateName = function(bookmark){

	    		//	Watch this variable
	    		var isValid = true;

	    		//	Bookmark must have something
	    		if (bookmark.name.trim().length == 0){
	    			isValid = false;
	    		}

	    		//	Loop through existing bookmarks
	    		var existing = $$get(prism.bookmarks, "saved",{});
	    		$.each(existing, function(){

	    			//	This only applies to bookmarks for this user
	    			if (this.createdBy == prism.user.email) {
	    				//	Has this user already created a bookmark with this name
		    			var isDuplicate = (this.name.toUpperCase() == bookmark.name.toUpperCase()) && (this.isPublic == bookmark.isPublic)
		    			if (isDuplicate){		    				
		    				isValid = false;
		    			}
		    		}
	    		})

	    		//	Update the variable
	    		if (isValid) {
	    			myScope.isValidName = true;
	    		} else {
	    			myScope.isValidName = false;
	    		}
	    	}

	    	//	Function to toggle a new bookmark between public/private
	    	myScope.setPublic = function(isPublic) {

	    		myScope.newBookmark.isPublic = isPublic;
	    	}

	    	//	Function to format the display name of a bookmark
	    	myScope.getName = function(bookmark) {
	    		//	Is the bookmark public?
	    		if (bookmark.isPublic){
	    			//	Public bookmark, who created it?
	    			if (prism.user.email == bookmark.createdBy) {
	    				//	This is the current user's bookmark, just say 'public'
	    				return bookmark.name + " (Public)";
	    			} else {
	    				//	This is a shared bookmark, indicate who created it
	    				return bookmark.name + " (" + bookmark.createdBy + ")";
	    			}
	    		} else {
	    			//	Private bookmark, just show the name
	    			return bookmark.name;
	    		}
	    	}

	    	//	Function that determines if a bookmark can be deleted
	    	myScope.canDelete = function(bookmark){
	    		if (prism.user.email == bookmark.createdBy){
	    			return true;
	    		} else {
	    			return false;
	    		}
	    	}
	    	
	    	//	Function to load a given bookmark
	    	myScope.loadBookmark = function (bookmark) {
	    		//	Set the bookmark
	        	prism.bookmarks.loadBookmark(bookmark);

	        	//	Close the window
	        	this.close();
	        };

	        //	Function to save a selected bookmark
	    	myScope.saveBookmark = function () {
	    		//	Validate the bookmark name  
	    		myScope.validateName(this.newBookmark); 	
	    		if (myScope.isValidName){	

	    			//	Save the bookmark
	        		prism.bookmarks.saveBookmark(this.newBookmark);

		        	//	Close the window
		        	this.close();
		        }
	        };

	        //	Function to delete a selected bookmark
	    	myScope.deleteBookmark = function (bookmark) {
	    		//	Delete the bookmark	    		
	    		if (prism.user.email == bookmark.createdBy){
	        		prism.bookmarks.deleteBookmark(bookmark,myScope);
	        	}
	        };

	        //	Function to close the window
	        myScope.close = function(){
	        	//	Run the close function
	        	this.finished()
	        };
    }])
    
	//	Add handler for header menu buttons
	mod.directive('filtersHeadline',[function(){
		return {
			restrict: 'C',
			replace: false,
			link: function(scope,element,attrs){				

				//	Get the filter label
				var headerLabel = parseFloat(prism.version) >= 6.6 ? $('.filters-title') : $('h2',element);

				//	Make it clickable				
				$(headerLabel).on('click', { scope: scope }, prism.bookmarks.popup);

				//	Add style for mouse pointer
				$(headerLabel).css('cursor', 'pointer');

				//	Add styling for on hover
				var hoverColor = "#ffff9c";
				var defaultColor = "transparent";
				$(headerLabel).mouseenter(function() {
				    $(this).css("background", hoverColor);
				}).mouseleave(function() {
				     $(this).css("background", defaultColor);
				});
			}
		}
	}])

	//	Add handler for filters menu
	prism.on('beforemenu',function(e, args){

		//	Only add for the filters menu
		if (args.settings.name === "filters") {

			//  Get the list of existing items            
            var items = args.settings.items;

            //  Create a separator
            var separator = {
                type: "separator"
            };

            //  Create a button for the bookmarks popup
            var button = {
                caption: 'Bookmarks',
                type: '"header"',
                execute: prism.bookmarks.popup,
                closing: true
            }

            //   Add options to the menu         
            args.settings.items.push(separator);
            args.settings.items.push(button);
		}
	})
}])